from AccessControl import ClassSecurityInfo
from ExtensionClass import Base
from OFS.Image import File
from Products.Archetypes.Field import FileField
from Products.Archetypes.Field import ObjectField, encode, decode
from Products.Archetypes import config
from Products.Archetypes.Registry import registerField
from Products.CMFCore.utils import getToolByName
from widget import TramlineWidget
import types
from zope import interface
from interfaces import ITramlineFile
import os
from stat import ST_SIZE

class TramlineFile(File):
    interface.implements(ITramlineFile)
    def __init__(self, id, title, file, content_type='', precondition='',
                 tramlined=False):
        File.__init__(self, id, title, file, content_type, precondition)
        self.is_tramlined = tramlined

    def index_html(self, REQUEST, RESPONSE):
        """Default view method with Tramline headers."""
        if self.is_tramlined:
            RESPONSE.setHeader('tramline_file','')
        return File.index_html(self, REQUEST, RESPONSE)

class TramlineField(FileField):
    security  = ClassSecurityInfo()

    _properties = ObjectField._properties.copy()
    _properties.update({
        'widget' : TramlineWidget,
        'content_class' : TramlineFile,
        'default_content_type' : 'application/octet-stream',
        })

    def notifyBeforeDelete(self, item, container):
        tramline_tool = getToolByName(item, 'portal_tramline')
        tramline_tool.removeFile(
                self.get(item, raw=True, unwrapped=True))

    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        """
        Assign input value to object. If mimetype is not specified,
        pass to processing method without one and add mimetype returned
        to kwargs. Assign kwargs to instance.
        """
        if isinstance(value, types.TupleType) or isinstance(
                value, types.ListType):
            tramlined = value[1]
            value = value[0]
        else:
            tramlined = False

        if value == "DELETE_FILE":
            if hasattr(instance, '_FileField_types'):
                delattr(aq_base(instance), '_FileField_types')
            ObjectField.unset(self, instance, **kwargs)
            return

        if not kwargs.has_key('mimetype'):
            kwargs['mimetype'] = None

        kwargs['default'] = self.getDefault(instance)
        initializing = kwargs.get('_initializing_', False)

        if not initializing:
            file = self.get(instance, raw=True, unwrapped=True)
        else:
            file = None
        factory = self.content_class
        if not initializing and not isinstance(file, factory):
            # Convert to same type as factory
            # This is here mostly for backwards compatibility
            v, m, f = self._migrate_old(file, **kwargs)
            kwargs['mimetype'] = m
            kwargs['filename'] = f
            obj = self._wrapValue(instance, v, **kwargs)
            # Store so the object gets a _p_jar,
            # if we are using a persistent storage, that is.
            ObjectField.set(self, instance, obj, **kwargs)
            file = self.get(instance, raw=True, unwrapped=True)
            # Should be same as factory now, but if it isn't, that's
            # very likely a bug either in the storage implementation
            # or on the field implementation.

        value, mimetype, filename = self._process_input(value, file=file,
                                                        instance=instance,
                                                        tramlined=tramlined,
                                                        **kwargs)
        kwargs['mimetype'] = mimetype
        kwargs['filename'] = filename

        # remove ugly hack
        if hasattr(instance, '_FileField_types'):
            del instance._FileField_types
        if value is None:
            # do not send None back as file value if we get a default (None)
            # value back from _process_input.  This prevents
            # a hard error (NoneType object has no attribute 'seek') from
            # occurring if someone types in a bogus name in a file upload
            # box (at least under Mozilla).
            value = ''
        obj = self._wrapValue(instance, value, **kwargs)
        obj.is_tramlined = tramlined
        ObjectField.set(self, instance, obj, **kwargs)

    security.declarePublic('get_size')
    def get_size(self, instance):
        """Get size of the stored data used for get_size in BaseObject
        """
        tramline_id = str(self.get(instance))
        tramline_tool = getToolByName(instance, 'portal_tramline')
        tramline_path = tramline_tool.getTramlinePath()

        try:
            file = os.path.join(tramline_path, 'repository', tramline_id)
            size = os.stat(file)[ST_SIZE]
        except OSError:
            try:
                file = os.path.join(tramline_path, 'upload', tramline_id)
                size = os.stat(file)[ST_SIZE]
            except OSError:
                size = 0

        return size
