import os

from Globals import package_home
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.public import listTypes
from Products.attramline.config import PROJECTNAME
from Products.CMFPlone.tests import PloneTestCase
from StringIO import StringIO
from Testing import ZopeTestCase

ZopeTestCase.installProduct('Five')
ZopeTestCase.installProduct('attramline')

class ATTramlineTests(PloneTestCase.PloneTestCase):
    def afterSetUp(self):
        self.portal.portal_quickinstaller.installProducts(['attramline'])

        classes = listTypes(PROJECTNAME)
        out = StringIO()
        installTypes(self.portal, out,
                     classes,
                     PROJECTNAME)

        PACKAGE_HOME = package_home(globals())
        self.path = os.path.join(os.path.join(
            *os.path.split(PACKAGE_HOME)[:-1]), os.path.join(
                'tests', 'sample.png'))

    def createContent(self, id):
        self.folder.invokeFactory('TypeWithTramlineSupport', id)
        return getattr(self.folder, id)

    def testTramlineDetection(self):
        # test with empy request
        doc = self.createContent('tramlinedetection')
        field = doc.getField('tramlineFile')
        value = field.widget.process_form(doc, field, {})
        self.assertEqual(value, None)

        # test with proper request
        value = field.widget.process_form(doc, field,
                                          {'tramlineFile_file': open(self.path),
                                           'tramline_enable': 'tramlineFile',
                                           'tramline':''})
        self.assertEqual(value[0][1], True)

        # test without tramline header
        value = field.widget.process_form(doc, field,
                                          {'tramlineFile_file': open(self.path),
                                           'tramline_enable': 'tramlineFile'})
        self.assertEqual(value[0][1], False)

        # test without tramline enable
        value = field.widget.process_form(doc, field,
                                          {'tramlineFile_file': open(self.path),
                                           'tramline':''})
        self.assertEqual(value[0][1], False)

    def testFileCreation(self):
        doc = self.createContent('tramlinefield')
        field = doc.getField('tramlineFile')
        doc.setTramlineFile(open(self.path))
        self.assertEqual(doc.getTramlineFile().is_tramlined, False)

        doc.setTramlineFile((open(self.path), False))
        self.assertEqual(doc.getTramlineFile().is_tramlined, False)
    
        doc.setTramlineFile((open(self.path), True))
        self.assertEqual(doc.getTramlineFile().is_tramlined, True)

    def testRemoval(self):
        
        doc = self.createContent('tramlineremoval')
        self.folder.manage_delObjects(ids=['tramlineremoval',])

        doc = self.createContent('tramlineremoval')
        doc.setTramlineFile((open(self.path), False))
        self.folder.manage_delObjects(ids=['tramlineremoval',])

        tramlinedir = os.path.join(os.tmpnam(), 'repository')
        # setup tool
        self.portal.portal_tramline.manage_changeProperties(
                tramlinepath=os.path.dirname(tramlinedir))
        
        os.makedirs(tramlinedir)
        tramlinefile = os.path.join(tramlinedir, 'someid')
        open(tramlinefile, 'w')
        doc = self.createContent('tramlineremoval')
        doc.setTramlineFile(('someid', True))
        self.folder.manage_delObjects(ids=['tramlineremoval',])
        self.failIf(os.path.exists(tramlinefile))

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(ATTramlineTests))
    return suite
